import org.junit.Assert;
import org.junit.Before;
import org.junit.Test;

import java.util.ArrayList;

public class ArrazTest {

    private Arraz arraz;
    private int[] intArray;
    private int[] intArray2;
    private double[] doubleArray;
    private double[] doubleArray2;

    @Before
    public void setUp() {
        arraz = new Arraz();
        intArray = new int[]{1, 2, 3, 4, 5};
        intArray2 = new int[]{5, 6, 7, 8, 9, 10};
        doubleArray = new double[]{11, 12, 13, 14, 15};
        doubleArray2 = new double[]{15, 16, 17, 18, 19, 20};
    }

    @Test
    public void sumValuesOfArray1() {
        int expected = 15;
        int actual = arraz.sumValuesOfArray(intArray);

        Assert.assertEquals(expected, actual);
    }

    @Test
    public void sumValuesOfArray2() {
        int expected = 45;
        int actual = arraz.sumValuesOfArray(intArray2);

        Assert.assertEquals(expected, actual);
    }

    //////////    //////////    //////////

    @Test
    public void sumDoubleOfArray1() {
        double expected = 65;
        double actual = arraz.sumDoubleOfArray(doubleArray);

        Assert.assertEquals(expected, actual, 1e-15);
    }

    @Test
    public void sumDoubleOfArray2() {
        double expected = 105;
        double actual = arraz.sumDoubleOfArray(doubleArray2);

        Assert.assertEquals(expected, actual, 1e-15);
    }

    //////////    //////////    //////////

    @Test
    public void averageOfArray1() {
        int expected = 3;
        int actual = arraz.averageOfArray(intArray);

        Assert.assertEquals(expected, actual);
    }

    @Test
    public void averageOfArray2() {
        int expected = 7;
        int actual = arraz.averageOfArray(intArray2);

        Assert.assertEquals(expected, actual);
    }

    //////////    //////////    //////////

    @Test
    public void doubleAverageOfArray1() {
        double expected = 13;
        double actual = arraz.doubleAverageOfArray(doubleArray);

        Assert.assertEquals(expected, actual, 1e-15);
    }

    @Test
    public void doubleAverageOfArray2() {
        double expected = 17.5;
        double actual = arraz.doubleAverageOfArray(doubleArray2);

        Assert.assertEquals(expected, actual, 1e-15);
    }

    //////////    //////////    //////////

    @Test
    public void containsValue1() {
        Assert.assertTrue(arraz.containsValue(intArray, 5));
        Assert.assertTrue(arraz.containsValue(intArray2, 5));
    }

    @Test
    public void containsValue2() {
        Assert.assertFalse(arraz.containsValue(intArray, 6));
        Assert.assertFalse(arraz.containsValue(intArray2, 4));
    }

    //////////    //////////    //////////

    @Test
    public void reverseArray1() {
        int[] expected = {5, 4, 3, 2, 1};
        int[] actual = arraz.reverseArray(intArray);

        for (int i = 0; i < expected.length; i++) {
            Assert.assertEquals(expected[i], actual[i]);
        }
    }

    @Test
    public void reverseArray2() {
        int[] expected = {10, 9, 8, 7, 6, 5};
        int[] actual = arraz.reverseArray(intArray2);

        for (int i = 0; i < expected.length; i++) {
            Assert.assertEquals(expected[i], actual[i]);
        }
    }

    //////////    //////////    //////////

    @Test
    public void getOddEvensOfArray1() {
        OddEven expected = new OddEven(3, 2);
        OddEven actual = arraz.getOddEvensOfArray(intArray);

        Assert.assertEquals(expected.odds, actual.odds);
        Assert.assertEquals(expected.evens, actual.evens);
    }

    @Test
    public void getOddEvensOfArray2() {
        OddEven expected = new OddEven(3, 3);
        OddEven actual = arraz.getOddEvensOfArray(intArray2);

        Assert.assertEquals(expected.odds, actual.odds);
        Assert.assertEquals(expected.evens, actual.evens);
    }

    //////////    //////////    //////////

    @Test
    public void findIndexOf1() {
        int expected = 3;
        int actual = arraz.findIndexOf(intArray, 4);

        Assert.assertEquals(expected, actual);
    }

    @Test
    public void findIndexOf2() {
        int expected = -1;
        int actual = arraz.findIndexOf(intArray2, 1);

        Assert.assertEquals(expected, actual);
    }

    //////////    //////////    //////////

    @Test
    public void createEmptyGenericArray() {
        String[] expected = new String[]{"this", "is", "a", "good", "test"};
        String[] actual = (String[]) arraz.createEmptyGenericArray(expected, expected.length);
        actual[0] = expected[0];

        Assert.assertEquals(expected.getClass(), actual.getClass());
        Assert.assertEquals(expected[0].getClass(), actual[0].getClass());
    }

    //////////    //////////    //////////

    @Test
    public void copyArrayByIterator1() {
        String[] expected = new String[]{"this", "is", "a", "good", "test"};
        String[] actual = (String[]) arraz.copyArrayByIterator(expected);

        for (int i = 0; i < expected.length; i++) {
            Assert.assertEquals(expected[i], actual[i]);
        }

        Assert.assertEquals(expected, actual);
    }

    @Test
    public void copyArrayByIterator2() {
        Integer[] expected = new Integer[]{1, 2, 3, 4, 5};
        Integer[] actual = (Integer[]) arraz.copyArrayByIterator(expected);

        for (int i = 0; i < expected.length; i++) {
            Assert.assertEquals(expected[i], actual[i]);
        }

        Assert.assertEquals(expected, actual);
    }

    //////////    //////////    //////////

    @Test
    public void copyArrayByLoop1() {
        String[] expected = new String[]{"this", "is", "a", "good", "test"};
        String[] actual = (String[]) arraz.copyArrayByLoop(expected);

        for (int i = 0; i < expected.length; i++) {
            Assert.assertEquals(expected[i], actual[i]);
        }

        Assert.assertEquals(expected, actual);
    }

    @Test
    public void copyArrayByLoop2() {
        Integer[] expected = new Integer[]{1, 2, 3, 4, 5};
        Integer[] actual = (Integer[]) arraz.copyArrayByLoop(expected);

        for (int i = 0; i < expected.length; i++) {
            Assert.assertEquals(expected[i], actual[i]);
        }

        Assert.assertEquals(expected, actual);
    }

    //////////    //////////    //////////

    @Test
    public void removeElementFromArray1() {
        String[] original = new String[]{"this", "is", "a", "good", "test"};

        String[] expected = new String[]{"this", "is", "a", "test"};
        String[] actual = (String[]) arraz.removeElementFromArray(original, "good");

        Assert.assertEquals(expected, actual);
    }

    @Test
    public void removeElementFromArray2() {
        Integer[] original = new Integer[]{1, 2, 3, 4, 5};

        Integer[] expected = new Integer[]{1, 2, 4, 5};
        Integer[] actual = (Integer[]) arraz.removeElementFromArray(original, 3);

        Assert.assertEquals(expected, actual);
    }

    //////////    //////////    //////////

    @Test
    public void insertIntoArrayAt1() {
        String[] original = new String[]{"this", "is", "a", "good", "test"};

        String[] expected = new String[]{"this", "is", "a", "good", "test", "!"};
        String[] actual = (String[]) arraz.insertIntoArrayAt(original, "!", 5);

        Assert.assertEquals(expected, actual);
    }

    @Test
    public void insertIntoArrayAt2() {
        Integer[] original = new Integer[]{1, 2, 3, 4, 5};

        Integer[] expected = new Integer[]{1, 2, 3, 100, 4, 5};
        Integer[] actual = (Integer[]) arraz.insertIntoArrayAt(original, 100, 3);

        Assert.assertEquals(expected, actual);
    }

    //////////    //////////    //////////

    @Test
    public void findMax() {
        int[] spiffyHandyIntArray = new int[]{4, 5, 102, 6, -7, 12, -32, 92, 8};

        int expected = 102;
        int actual = arraz.findMax(spiffyHandyIntArray);

        Assert.assertEquals(expected, actual);
    }

    @Test
    public void findMin() {
        int[] spiffyHandyIntArray = new int[]{4, 5, 102, 6, -7, 12, -32, 92, 8};

        int expected = -32;
        int actual = arraz.findMin(spiffyHandyIntArray);

        Assert.assertEquals(expected, actual);
    }

    //////////    //////////    //////////

    @Test
    public void findMaxMinOfArray1() {
        MaxMin expected = new MaxMin(5, 1);
        MaxMin actual = arraz.findMaxMinOfArray(intArray);

        Assert.assertEquals(expected.max, actual.max);
        Assert.assertEquals(expected.min, actual.min);
    }

    @Test
    public void findMaxMinOfArray2() {
        MaxMin expected = new MaxMin(10, 5);
        MaxMin actual = arraz.findMaxMinOfArray(intArray2);

        Assert.assertEquals(expected.max, actual.max);
        Assert.assertEquals(expected.min, actual.min);
    }

    //////////    //////////    //////////


    @Test
    public void removeDupesFromArray1() {
        Integer[] original = new Integer[]{2, 2, 1, 3, 4, 5, 5, 6, 2, 1, 8};

        Integer[] expected = new Integer[]{2, 1, 3, 4, 5, 6, 8};
        Integer[] actual = arraz.removeDupesFromArray(original);

        Assert.assertEquals(expected, actual);
    }

    @Test
    public void removeDupesFromArray2() {
        Integer[] original = new Integer[]{10, 6, 20, 22, 28, 5, 999, 5};

        Integer[] expected = new Integer[]{10, 6, 20, 22, 28, 5, 999};
        Integer[] actual = arraz.removeDupesFromArray(original);

        Assert.assertEquals(expected, actual);
    }

    //////////    //////////    //////////

    @Test
    public void find2ndLargestValueFromArray1() {
        Double[] original = new Double[]{2.4, 3.0, 6.5, 7.1, 2.8, 1.1, 0.5};

        Double expected = 6.5;
        Double actual = arraz.find2ndLargestValueFromArray(original);

        Assert.assertEquals(expected, actual, 1e-15);
    }


    @Test
    public void find2ndLargestValueFromArray2() {
        Double[] original = new Double[]{1.5, 1.3, 1.4, 1.2, 1.1};

        Double expected = 1.4;
        Double actual = arraz.find2ndLargestValueFromArray(original);

        Assert.assertEquals(expected, actual, 1e-15);
    }

    //////////    //////////    //////////

    @Test
    public void makeMeAnArrayListFromArray1() {
        Integer[] original = new Integer[]{1, 2, 3};

        ArrayList<Integer> expected = new ArrayList<>();
        expected.add(1);
        expected.add(2);
        expected.add(3);

        ArrayList<Integer> actual = arraz.makeMeAnArrayListFromArray(original);

        Assert.assertEquals(expected, actual);
    }

    @Test
    public void makeMeAnArrayListFromArray2() {
        String[] original = new String[]{"I", "am", "very", "hungry", "!"};

        ArrayList<String> expected = new ArrayList<>();
        expected.add("I");
        expected.add("am");
        expected.add("very");
        expected.add("hungry");
        expected.add("!");

        ArrayList<String> actual = arraz.makeMeAnArrayListFromArray(original);

        Assert.assertEquals(expected, actual);
    }

    //////////    //////////    //////////

    @Test
    public void makeMeAnArrayFromArrayList1() {
        ArrayList<Integer> original = new ArrayList<>();
        original.add(6);
        original.add(20);
        original.add(92);

        Integer[] expected = new Integer[]{6, 20, 92};
        Integer[] actual = new Integer[original.size()];
        actual = (Integer[]) arraz.makeMeAnArrayFromArrayList(original, actual);

        Assert.assertEquals(expected, actual);
    }

    @Test
    public void makeMeAnArrayFromArrayList2() {
        ArrayList<String> original = new ArrayList<>();
        original.add("hello");
        original.add("goodbye");

        String[] expected = new String[]{"hello", "goodbye"};
        String[] actual = new String[original.size()];
        actual = (String[]) arraz.makeMeAnArrayFromArrayList(original, new String[0]);

        Assert.assertEquals(expected, actual);
    }

    //////////    //////////    //////////

    @Test
    public void check2ArraysForEquals1() {
        String[] string1 = "I am starting to hate arrays".split(" ");
        String[] string2 = new String[]{"I", "am", "starting", "to", "hate", "arrays"};

        Assert.assertTrue(arraz.check2ArraysForEquals(string1, string2));
    }

    @Test
    public void check2ArraysForEquals2() {
        Double[] double1 = new Double[]{1.0, 0.5, 3.6, 38.4, 17.3, 62.0, 9.0, 3.375, 0.0, 3.14159};
        Double[] double2 = new Double[]{1.0, 0.5, 3.6, 38.4, 17.3, 62.1, 9.0, 3.375, 0.0, 3.14159};

        Assert.assertFalse(arraz.check2ArraysForEquals(double1, double2));
    }

    //////////    //////////    //////////

    @Test
    public void averageArrayWithoutMaxMin1() {
        String expected = "3.00";
        String actual = String.format("%.2f", arraz.averageArrayWithoutMaxMin(intArray));

        Assert.assertEquals(expected, actual);
    }

    @Test
    public void averageArrayWithoutMaxMin2() {
        int[] original = new int[]{9, 9, 2, 5, -18, 5, 2, 35, 291, 291, 291, -18};

        String expected = "9.57";
        String actual = String.format("%.2f", arraz.averageArrayWithoutMaxMin(original));

        Assert.assertEquals(expected, actual);
    }

    //////////    //////////    //////////

    @Test
    public void arrayHas65And77_1() {
        int[] original = new int[]{1, 1, 2, 65, 64, 98, 78};

        Assert.assertFalse(arraz.arrayHas65And77(original));
    }

    @Test
    public void arrayHas65And77_2() {
        int[] original = new int[]{1, 1, 2, 65, 64, 98, 77};

        Assert.assertTrue(arraz.arrayHas65And77(original));
    }

    //////////    //////////    //////////

    @Test
    public void theTotalofTensIs30_1() {
        int[] original = new int[]{10, 1, 2, 65, 10, 64, 10, 98, 77};

        Assert.assertTrue(arraz.theTotalofTensIs30(original));
    }

    @Test
    public void theTotalofTensIs30_2() {
        int[] original = new int[]{10, 1, 2, 10, 65, 10, 64, 10, 98, 77};

        Assert.assertFalse(arraz.theTotalofTensIs30(original));
    }

    //////////    //////////    //////////

    @Test
    public void findSmallestOfArray1() {
        int expected = 1;
        int actual = arraz.findSmallestOfArray(intArray);

        Assert.assertEquals(expected, actual);
    }

    @Test
    public void findSmallestOfArray2() {
        int[] spiffyHandyIntArray = new int[]{4, 5, 102, 6, -7, 12, -32, 92, 8};

        int expected = -32;
        int actual = arraz.findSmallestOfArray(spiffyHandyIntArray);

        Assert.assertEquals(expected, actual);
    }

    //////////    //////////    //////////

    @Test
    public void findSecondSmallestOfArray1() {
        int expected = 2;
        int actual = arraz.findSecondSmallestOfArray(intArray);

        Assert.assertEquals(expected, actual);
    }

    @Test
    public void findSecondSmallestOfArray2() {
        int[] spiffyHandyIntArray = new int[]{4, 5, 102, 6, -7, 12, -32, 92, 8};

        int expected = -7;
        int actual = arraz.findSecondSmallestOfArray(spiffyHandyIntArray);

        Assert.assertEquals(expected, actual);
    }


    //////////    //////////    //////////

    @Test
    public void makeMeACopyPlease1() {
        int[] expected = new int[]{5, 4, 3, 2, 1};
        int[] actual = arraz.makeMeACopyPlease(intArray);

        for (int i = 0; i < expected.length; i++) {
            Assert.assertEquals(expected[i], actual[i]);
        }
    }

    @Test
    public void makeMeACopyPlease2() {
        int[] spiffyHandyIntArray = new int[]{4, 5, 102, 6, -7, 12, -32, 92, 8};

        int[] expected = new int[]{8, 92, -32, 12, -7, 6, 102, 5, 4};
        int[] actual = arraz.makeMeACopyPlease(spiffyHandyIntArray);

        for (int i = 0; i < expected.length; i++) {
            Assert.assertEquals(expected[i], actual[i]);
        }
    }

    //////////    //////////    //////////

    @Test
    public void removeLastItemAndCopy1() {
        Integer[] original = new Integer[]{10, 6, 20, 22, 28, 5, 999, 5};

        Integer[] expected = new Integer[]{10, 6, 20, 22, 28, 5, 999};
        Integer[] actual = (Integer[]) arraz.removeLastItemAndCopy(original);

        Assert.assertEquals(expected, actual);
    }

    @Test
    public void removeLastItemAndCopy2() {
        String[] original = new String[]{"My", "name", "is", "Jenn", "Chao"};

        String[] expected = new String[]{"My", "name", "is", "Jenn"};
        String[] actual = (String[]) arraz.removeLastItemAndCopy(original);

        Assert.assertEquals(expected, actual);
    }

    //////////    //////////    //////////

    @Test
    public void removeFirstItemAndCopy1() {
        Integer[] original = new Integer[]{10, 6, 20, 22, 28, 5, 999, 5};

        Integer[] expected = new Integer[]{6, 20, 22, 28, 5, 999, 5};
        Integer[] actual = (Integer[]) arraz.removeFirstItemAndCopy(original);

        Assert.assertEquals(expected, actual);
    }

    @Test
    public void removeFirstItemAndCopy2() {
        String[] original = new String[]{"My", "name", "is", "Jenn", "Chao"};

        String[] expected = new String[]{"name", "is", "Jenn", "Chao"};
        String[] actual = (String[]) arraz.removeFirstItemAndCopy(original);

        Assert.assertEquals(expected, actual);
    }

    //////////    //////////    //////////

    @Test
    public void insertAtStartAndCopy1() {
        Integer[] original = new Integer[]{1, 2, 3, 4, 5};

        Integer[] expected = new Integer[]{99, 1, 2, 3, 4, 5};
        Integer[] actual = (Integer[]) arraz.insertAtStartAndCopy(original, 99);

        Assert.assertEquals(expected, actual);
    }

    @Test
    public void insertAtStartAndCopy2() {
        String[] original = new String[]{"I", "want", "fruit", "snacks"};

        String[] expected = new String[]{"Seriously", "I", "want", "fruit", "snacks"};
        String[] actual = (String[]) arraz.insertAtStartAndCopy(original, "Seriously");

        Assert.assertEquals(expected, actual);

    }

    //////////    //////////    //////////

    @Test
    public void insertAtEndAndCopy1() {
        Integer[] original = new Integer[]{1, 2, 3, 4, 5};

        Integer[] expected = new Integer[]{1, 2, 3, 4, 5, 99};
        Integer[] actual = (Integer[]) arraz.insertAtEndAndCopy(original, 99);

        Assert.assertEquals(expected, actual);

    }

    @Test
    public void insertAtEndAndCopy2() {
        String[] original = new String[]{"I", "want", "fruit", "snacks"};

        String[] expected = new String[]{"I", "want", "fruit", "snacks", "!"};
        String[] actual = (String[]) arraz.insertAtEndAndCopy(original, "!");

        Assert.assertEquals(expected, actual);
    }

    //////////    //////////    //////////

    @Test
    public void sortArrayIntoEvensThenOdds1() {
        int[] spiffyHandyIntArray = new int[] {4, 5, 102, 6, -7, 12, -32, 92, 8};

        ArrayList<Integer> expected = new ArrayList<>();
        expected.add(4); expected.add(102); expected.add(6); expected.add(12); expected.add(-32); expected.add(92); expected.add(8); expected.add(5); expected.add(-7);
        ArrayList<Integer> actual = arraz.sortArrayIntoEvensThenOdds(spiffyHandyIntArray);

        Assert.assertEquals(expected, actual);
    }

    @Test
    public void sortArrayIntoEvensThenOdds2() {
        ArrayList<Integer> expected = new ArrayList<>();
        expected.add(2); expected.add(4); expected.add(1); expected.add(3); expected.add(5);
        ArrayList<Integer> actual = arraz.sortArrayIntoEvensThenOdds(intArray);

        Assert.assertEquals(expected, actual);
    }
}